from setuptools import setup

setup(name='yahoo_ff',
      version='0.13',
      description='retrieve financial fundamental data for stocks',
      url='http://github.com/alexandresobolevski/yahoo_ff',
      author='Alexandre Sobolevski',
      author_email='',
      license='MIT',
      packages=['yahoo_ff'],
      zip_safe=False)
